﻿//////////////////////////////////////////////////////
////Addon			: Bounded Drag & Drop
////Type			: Behavior
////Version 		: 2.0.0.0
////Author			: Chadori_RebornXD
////Description		: An advanced version of Drag and drop. Drags & Drops objects either by mouse or touch with bounds.
////Dependency 		: None
////Distribution 	: Free
////Compatibility	: Construct 3 & Construct 2
////Help			: https://www.construct.net/make-games/addons/91/bounded-drag-drop
////Comment			: In case you need help, Google it.
//////////////////////////////////////////////////////

// ECMAScript 5 strict mode
"use strict";

assert2(cr, "cr namespace not created");
assert2(cr.behaviors, "cr.behaviors not created");

/////////////////////////////////////
// Behavior class
cr.behaviors.CV_BoundedDragnDrop = function(runtime)
{
	this.runtime = runtime;
	var self = this;

	//Chadori_RebornXD start	//For C3
	
	document.addEventListener("mousemove", function(info) {
		self.onMouseMove(info);
	});
		
	document.addEventListener("mousedown", function(info) {
		self.onMouseDown(info);
	});
	
	document.addEventListener("mouseup", function(info) {
		self.onMouseUp(info);
	});

	//Chadori_RebornXD end
	
	// Use document touch input for fullscreen mode
	var elem = (this.runtime.fullscreen_mode > 0) ? document : this.runtime.canvas;
	
	if (this.runtime.isDirectCanvas)
		elem = window["Canvas"];
	else if (this.runtime.isCocoonJs)
		elem = window;
	
	if (typeof PointerEvent !== "undefined")
	{
		elem.addEventListener("pointerdown",
			function(info) {
				self.onPointerStart(info);
			},
			false
		);
		
		elem.addEventListener("pointermove",
			function(info) {
				self.onPointerMove(info);
			},
			false
		);
		
		elem.addEventListener("pointerup",
			function(info) {
				self.onPointerEnd(info);
			},
			false
		);
		
		// Treat pointer cancellation the same as a touch end
		elem.addEventListener("pointercancel",
			function(info) {
				self.onPointerEnd(info);
			},
			false
		);
	}
	else if (window.navigator["msPointerEnabled"])
	{
		elem.addEventListener("MSPointerDown",
			function(info) {
				self.onPointerStart(info);
			},
			false
		);
		
		elem.addEventListener("MSPointerMove",
			function(info) {
				self.onPointerMove(info);
			},
			false
		);
		
		elem.addEventListener("MSPointerUp",
			function(info) {
				self.onPointerEnd(info);
			},
			false
		);
		
		// Treat pointer cancellation the same as a touch end
		elem.addEventListener("MSPointerCancel",
			function(info) {
				self.onPointerEnd(info);
			},
			false
		);
	}
	else
	{
		elem.addEventListener("touchstart",
			function(info) {
				self.onTouchStart(info);
			},
			false
		);
		
		elem.addEventListener("touchmove",
			function(info) {
				self.onTouchMove(info);
			},
			false
		);
		
		elem.addEventListener("touchend",
			function(info) {
				self.onTouchEnd(info);
			},
			false
		);
		
		elem.addEventListener("touchcancel",
			function(info) {
				self.onTouchEnd(info);
			},
			false
		);
	}
};

(function ()
{
	var behaviorProto = cr.behaviors.CV_BoundedDragnDrop.prototype;
	
	var dummyoffset = {left: 0, top: 0};
	
	function GetDragDropBehavior(inst)
	{
		var i, len;
		for (i = 0, len = inst.behavior_insts.length; i < len; i++)
		{
			if (inst.behavior_insts[i] instanceof behaviorProto.Instance)
				return inst.behavior_insts[i];
		}
		
		return null;
	};
	
	behaviorProto.onMouseDown = function (info)
	{
		if (info.which !== 1)
			return;		// not left mouse button
			
		this.onInputDown("leftmouse", info.pageX, info.pageY);
	};
	
	behaviorProto.onMouseMove = function (info)
	{
		if (info.which !== 1)
			return;		// not left mouse button
			
		this.onInputMove("leftmouse", info.pageX, info.pageY);
	};
	
	behaviorProto.onMouseUp = function (info)
	{
		if (info.which !== 1)
			return;		// not left mouse button
			
		this.onInputUp("leftmouse");
	};
	
	behaviorProto.onTouchStart = function (info)
	{
		if (info.preventDefault && cr.isCanvasInputEvent(info))
			info.preventDefault();
		
		var i, len, t, id;
		for (i = 0, len = info.changedTouches.length; i < len; i++)
		{
			t = info.changedTouches[i];
			
			// directCanvas does not send an identifier
			id = t.identifier;
			this.onInputDown(id ? id.toString() : "<none>", t.pageX, t.pageY);
		}
	};
	
	behaviorProto.onTouchMove = function (info)
	{
		if (info.preventDefault)
			info.preventDefault();
		
		var i, len, t, id;
		for (i = 0, len = info.changedTouches.length; i < len; i++)
		{
			t = info.changedTouches[i];
			id = t.identifier;
			this.onInputMove(id ? id.toString() : "<none>", t.pageX, t.pageY);
		}
	};
	
	behaviorProto.onTouchEnd = function (info)
	{
		if (info.preventDefault && cr.isCanvasInputEvent(info))
			info.preventDefault();
		
		var i, len, t, id;
		for (i = 0, len = info.changedTouches.length; i < len; i++)
		{
			t = info.changedTouches[i];
			id = t.identifier;
			this.onInputUp(id ? id.toString() : "<none>");
		}
	};
	
	behaviorProto.onPointerStart = function (info)
	{
		// Ignore mouse events
		if (info["pointerType"] === info["MSPOINTER_TYPE_MOUSE"] || info["pointerType"] === "mouse")
			return;
			
		if (info.preventDefault && cr.isCanvasInputEvent(info))
			info.preventDefault();
		
		this.onInputDown(info["pointerId"].toString(), info.pageX, info.pageY);
	};
	
	behaviorProto.onPointerMove = function (info)
	{
		// Ignore mouse events
		if (info["pointerType"] === info["MSPOINTER_TYPE_MOUSE"] || info["pointerType"] === "mouse")
			return;
			
		if (info.preventDefault)
			info.preventDefault();
		
		this.onInputMove(info["pointerId"].toString(), info.pageX, info.pageY);
	};
	
	behaviorProto.onPointerEnd = function (info)
	{
		// Ignore mouse events
		if (info["pointerType"] === info["MSPOINTER_TYPE_MOUSE"] || info["pointerType"] === "mouse")
			return;
			
		if (info.preventDefault && cr.isCanvasInputEvent(info))
			info.preventDefault();
		
		this.onInputUp(info["pointerId"].toString());
	};
	
	behaviorProto.onInputDown = function (src, pageX, pageY)
	{
		//Chadori_RebornXD start	//For C3
		var offsetLeft = this.runtime.canvas.offsetLeft;
		var offsetTop = this.runtime.canvas.offsetTop;
		//Chadori_RebornXD end
		var x = pageX - offsetLeft;
		var y = pageY - offsetTop;
		var lx, ly, topx, topy;
		
		var arr = this.my_instances.valuesRef();
		
		var i, len, b, inst, topmost = null;
		for (i = 0, len = arr.length; i < len; i++)
		{
			inst = arr[i];
			b = GetDragDropBehavior(inst);
			
			if (!b.enabled || b.dragging)
				continue;		// don't consider disabled or already-dragging instances
				
			lx = inst.layer.canvasToLayer(x, y, true);
			ly = inst.layer.canvasToLayer(x, y, false);
			inst.update_bbox();
			if (!inst.contains_pt(lx, ly))
				continue;		// don't consider instances not over this point
				
			// First instance found
			if (!topmost)
			{
				topmost = inst;
				topx = lx;
				topy = ly;
				continue;
			}
			
			// Otherwise prefer the topmost instance of all overlapping the point
			if (inst.layer.index > topmost.layer.index)
			{
				topmost = inst;
				topx = lx;
				topy = ly;
				continue;
			}
			
			if (inst.layer.index === topmost.layer.index && inst.get_zindex() > topmost.get_zindex())
			{
				topmost = inst;
				topx = lx;
				topy = ly;
				continue;
			}
		}
		
		if (topmost)
			GetDragDropBehavior(topmost).onDown(src, topx, topy);
	};
	
	behaviorProto.onInputMove = function (src, pageX, pageY)
	{
		//Chadori_RebornXD start	//For C3
		var offsetLeft = this.runtime.canvas.offsetLeft;
		var offsetTop = this.runtime.canvas.offsetTop;
		//Chadori_RebornXD end
		var x = pageX - offsetLeft;
		var y = pageY - offsetTop;
		var lx, ly;

		var arr = this.my_instances.valuesRef();
		
		var i, len, b, inst;
		for (i = 0, len = arr.length; i < len; i++)
		{
			inst = arr[i];
			b = GetDragDropBehavior(inst);
			
			if (!b.enabled || !b.dragging || (b.dragging && b.dragsource !== src))
				continue;		// don't consider disabled, not-dragging, or dragging by other sources
				
			lx = inst.layer.canvasToLayer(x, y, true);
			ly = inst.layer.canvasToLayer(x, y, false);
			b.onMove(lx, ly);
		}
	};
	
	behaviorProto.onInputUp = function (src)
	{
		var arr = this.my_instances.valuesRef();
		
		var i, len, b, inst;
		for (i = 0, len = arr.length; i < len; i++)
		{
			inst = arr[i];
			b = GetDragDropBehavior(inst);
			
			if (b.dragging && b.dragsource === src)
				b.onUp();
		}
	};
		
	/////////////////////////////////////
	// Behavior type class
	behaviorProto.Type = function(behavior, objtype)
	{
		this.behavior = behavior;
		this.objtype = objtype;
		this.runtime = behavior.runtime;
	};
	
	var behtypeProto = behaviorProto.Type.prototype;

	behtypeProto.onCreate = function()
	{
	};

	/////////////////////////////////////
	// Behavior instance class
	behaviorProto.Instance = function(type, inst)
	{
		this.type = type;
		this.behavior = type.behavior;
		this.inst = inst;				// associated object instance to modify
		this.runtime = type.runtime;
	};
	
	var behinstProto = behaviorProto.Instance.prototype;

	behinstProto.onCreate = function()
	{
		this.dragging = false;
		this.dx = 0;
		this.dy = 0;
		this.dragsource = "<none>";
		
		// 0 = both, 1 = horizontal, 2 = vertical
		this.axes = this.properties[0];
		this.enabled = (this.properties[1] !== 0);

		//Chadori_RebornXD start
			//user-defined
		this.boundTo = this.properties[2];
		this.boundBy = this.properties[3];
		this.boundMethod = this.properties[4];
		this.boundXRadius = this.properties[5];
		this.boundYRadius = this.properties[6];
		this.boundMinX = this.properties[7];
		this.boundMaxX = this.properties[8];
		this.boundMinY = this.properties[9];
		this.boundMaxY = this.properties[10];
			//background
		this.boundEnabled = false;
		this.radiusBound = false;
		this.posBound = false;
		this.posX = this.inst.x;
		this.posY = this.inst.y;
			//dedicated for expressions
		this.radiusExceeded = false;
		this.positionExceeded = false;

		this.getBoundValues = function ()
		{

			//Off
			if (this.boundTo === 0)
			{
					//All (false)
					this.boundEnabled = false;
					this.radiusBound = false;
					this.posBound = false;	
						
			}

			//Both
			else if (this.boundTo === 1)
			{

				//Enable Bound (true)
				this.boundEnabled = true;

				//Radius (true)
				if (this.boundXRadius !== 0 || this.boundYRadius !== 0)	
				{
					this.radiusBound = true;

					//XAxis Adjust
					if(this.boundXRadius === 0)
					{
						if (this.axes === 0 || this.axes === 1)
						{
							this.axes = 2;
						}
					}
					//YAxis Adjust
					if(this.boundYRadius === 0)
					{
						if (this.axes === 0 || this.axes === 2)
						{
							this.axes = 1;
						}
					}


				}

				else 
				{
					this.radiusBound = false;
				}


				//Position (true)
					this.posBound = true;

			}



			else if (this.boundTo === 2)
			{

				if (this.boundXRadius !== 0 || this.boundYRadius !== 0)
				
				{
					//Enable Bound (true)
					this.boundEnabled = true;

					//Radius (true)
					if (this.boundXRadius !== 0 || this.boundYRadius !== 0)	
					{

						this.radiusBound = true;


						//XAxis Adjust
						if(this.boundXRadius === 0)
						{
							if (this.axes === 0 || this.axes === 1)
							{
								this.axes = 2;
							}
						}
						//YAxis Adjust
						if(this.boundYRadius === 0)
						{
							if (this.axes === 0 || this.axes === 2)
							{
								this.axes = 1;
							}
						}


					}

					else 
					{
						this.radiusBound = false;
					}



					//Position (false)
					this.posBound = false;

				}		
			}

			else // (this.boundTo === 3)
			{
					//Enable Bound (true)
					this.boundEnabled = true;

					//radius (false)
					this.radiusBound = false;

					//position (true)
					this.posBound = true;
			}


		}

		this.getBoundValues();

		//Chadori_RebornXD end

	};
	
	behinstProto.saveToJSON = function ()
	{
		return { "enabled": this.enabled };
	};
	
	behinstProto.loadFromJSON = function (o)
	{
		this.enabled = o["enabled"];
		this.dragging = false;
	};
	
	behinstProto.onDown = function(src, x, y)
	{
		this.dx = x - this.inst.x;
		this.dy = y - this.inst.y;
		this.dragging = true;
		this.dragsource = src;
		
		// Trigger 'On drag start'
		this.runtime.isInUserInputEvent = true;
		this.runtime.trigger(cr.behaviors.CV_BoundedDragnDrop.prototype.cnds.OnDragStart, this.inst);
		this.runtime.isInUserInputEvent = false;
	};
	
	behinstProto.onMove = function(x, y)
	{
		var newx = x - this.dx;
		var newy = y - this.dy;
		
		//Chadori_RebornXD start

		if (this.axes === 0)		// both
		{

			//If one or more Bound values aren't 0.
			if (this.boundEnabled)	
			{
				//origin
				if (this.boundBy === 0)
				{
					//Radius
					this.limitLowX = this.posX - this.boundXRadius;
					this.limitHighX = this.posX + this.boundXRadius;
					this.limitLowY = this.posY - this.boundYRadius;
					this.limitHighY = this.posY + this.boundYRadius;
					//Position
					this.limitLowPosX = this.boundMinX;
					this.limitHighPosX = this.boundMaxX;
					this.limitLowPosY = this.boundMinY;
					this.limitHighPosY = this.boundMaxY;
					//Boolean
					var radiusClear = false;
					var posClear = false;

					//Set Clearance
					//radiusBound clearance
					if (this.radiusBound)
					{
						if (newx < this.limitLowX || newx > this.limitHighX || newy < this.limitLowY || newy > this.limitHighY)
						{
							radiusClear = false; //not necessary (redundant)
							
							//radius Exceeded
							this.radiusExceeded = true;

							//OnAnyBoundExceeded conditional trigger
							this.runtime.trigger(cr.behaviors.CV_BoundedDragnDrop.prototype.cnds.OnAnyBoundExceeded, this.inst);

							//OnRadiusExceeded conditional trigger
							this.runtime.trigger(cr.behaviors.CV_BoundedDragnDrop.prototype.cnds.OnRadiusExceeded, this.inst);
							
						}
						
						else 
						{

							radiusClear = true;

							//radius !Exceeded
							this.radiusExceeded = false;
						}
					
					}

					else
					{
						radiusClear = true;

						//radius !Exceeded
						this.radiusExceeded = false;

					}

					//posBound clearance
					if (this.posBound)
					{
						if (newx < this.limitLowPosX || newx > this.limitHighPosX || newy < this.limitLowPosY || newy > this.limitHighPosY)
						{
							posClear = false; //not necessary (redundant)

							//position Exceeded
							this.positionExceeded = true;

							//OnAnyBoundExceeded conditional trigger
							this.runtime.trigger(cr.behaviors.CV_BoundedDragnDrop.prototype.cnds.OnAnyBoundExceeded, this.inst);

							//OnPositionExceeded conditional trigger
							this.runtime.trigger(cr.behaviors.CV_BoundedDragnDrop.prototype.cnds.OnPositionExceeded, this.inst);

						}
						
						else 
						{

							posClear = true;

							//position !Exceeded
							this.positionExceeded = false;
						}
					
					}

					else
					{
						posClear = true;

						//position !Exceeded
						this.positionExceeded = false;

					}


					//Set New Position
					//BoundMethod = 'Halt'
					if (this.boundMethod === 0)
					{

						if (radiusClear && posClear)
						{
							if (this.inst.x !== newx || this.inst.y !== newy)	
							{					
								this.inst.x = newx;
								this.inst.y = newy;
								this.inst.set_bbox_changed();
				
							}
						}

					}
					//BoundMethod = 'Clamp'
					else
					{
						//!RadiusClear
						if (!radiusClear)
						{
							//RadiusX
							if (newx < this.limitLowX)
							{

								newx = this.limitLowX;

							}
							
							if (newx > this.limitHighX)
							{

								newx = this.limitHighX;

							}
							//RadiusY
							if (newy < this.limitLowY)
							{

								newy = this.limitLowY;

							}
							
							if (newy > this.limitHighY)
							{

								newy = this.limitHighY;

							}

						}
						//!PosClear
						if (!posClear)
						{
							//PosX
							if (newx < this.limitLowPosX)
							{

								newx = this.limitLowPosX;

							}
							
							if (newx > this.limitHighPosX)
							{

								newx = this.limitHighPosX;

							}
							//PosY
							if (newy < this.limitLowPosY)
							{

								newy = this.limitLowPosY;

							}
							
							if (newy > this.limitHighPosY)
							{

								newy = this.limitHighPosY;

							}

						}

						//New Position

						if (this.inst.x !== newx || this.inst.y !== newy)	
							{					
								this.inst.x = newx;
								this.inst.y = newy;
								this.inst.set_bbox_changed();
				
							}

					}
				}

				//edge
				else 

				{
					//Hotspot
					var hotspotX = this.inst.hotspotX;
					var hotspotY = this.inst.hotspotY;

					var hotspotWidthLeft = (hotspotX * this.inst.width);
					var hotspotWidthRight = ((1 - hotspotX) * this.inst.width);

					var hotspotHeightTop = (hotspotY * this.inst.height);
					var hotspotHeightBottom = ((1 - hotspotY) * this.inst.height);

					//Radius
					this.limitLowX = (this.posX + hotspotWidthLeft) - this.boundXRadius;
					this.limitHighX = (this.posX - hotspotWidthRight) + this.boundXRadius;
					this.limitLowY = (this.posY + hotspotHeightTop) - this.boundYRadius;
					this.limitHighY = (this.posY - hotspotHeightBottom) + this.boundYRadius;
					//Position
					this.limitLowPosX = this.boundMinX + hotspotWidthLeft;
					this.limitHighPosX = this.boundMaxX - hotspotWidthRight;
					this.limitLowPosY = this.boundMinY + hotspotHeightTop;
					this.limitHighPosY = this.boundMaxY - hotspotHeightBottom;
					//Boolean
					var radiusClear = false;
					var posClear = false;
					

					//Set New Position

					//Set Clearance
					//radiusBound clearance
					if (this.radiusBound)
					{
						if (newx < this.limitLowX || newx > this.limitHighX || newy < this.limitLowY || newy > this.limitHighY)
						{
							radiusClear = false; //not necessary (redundant)

							//radius Exceeded
							this.radiusExceeded = true;

							//OnAnyBoundExceeded conditional trigger
							this.runtime.trigger(cr.behaviors.CV_BoundedDragnDrop.prototype.cnds.OnAnyBoundExceeded, this.inst);

							//OnRadiusExceeded conditional trigger
							this.runtime.trigger(cr.behaviors.CV_BoundedDragnDrop.prototype.cnds.OnRadiusExceeded, this.inst);

						}
						
						else 
						{

							radiusClear = true;

							//radius !Exceeded
							this.radiusExceeded = false;

						}
					
					}

					else
					{
						radiusClear = true;

						//radius !Exceeded
						this.radiusExceeded = false;

					}

					//posBound clearance
					if (this.posBound)
					{
						if (newx < this.limitLowPosX || newx > this.limitHighPosX || newy < this.limitLowPosY || newy > this.limitHighPosY)
						{
							posClear = false; //not necessary (redundant)

							//position Exceeded
							this.positionExceeded = true;

							//OnAnyBoundExceeded conditional trigger
							this.runtime.trigger(cr.behaviors.CV_BoundedDragnDrop.prototype.cnds.OnAnyBoundExceeded, this.inst);

							//OnPositionExceeded conditional trigger
							this.runtime.trigger(cr.behaviors.CV_BoundedDragnDrop.prototype.cnds.OnPositionExceeded, this.inst);

						}
						
						else 
						{

							posClear = true;

							//position !Exceeded
							this.positionExceeded = false;

						}
					
					}

					else
					{
						posClear = true;

						//position !Exceeded
						this.positionExceeded = false;

					}


					//Set New Position
					//BoundMethod = 'Halt'
					if (this.boundMethod === 0)
					{

						if (radiusClear && posClear)
						{
							if (this.inst.x !== newx || this.inst.y !== newy)	
							{					
								this.inst.x = newx;
								this.inst.y = newy;
								this.inst.set_bbox_changed();
				
							}
						}

					}
					//BoundMethod = 'Clamp'
					else
					{
						//!RadiusClear
						if (!radiusClear)
						{
							//RadiusX
							if (newx < this.limitLowX)
							{

								newx = this.limitLowX;

							}
							
							if (newx > this.limitHighX)
							{

								newx = this.limitHighX;

							}
							//RadiusY
							if (newy < this.limitLowY)
							{

								newy = this.limitLowY;

							}
							
							if (newy > this.limitHighY)
							{

								newy = this.limitHighY;

							}

						}
						//!PosClear
						if (!posClear)
						{
							//PosX
							if (newx < this.limitLowPosX)
							{

								newx = this.limitLowPosX;

							}
							
							if (newx > this.limitHighPosX)
							{

								newx = this.limitHighPosX;

							}
							//PosY
							if (newy < this.limitLowPosY)
							{

								newy = this.limitLowPosY;

							}
							
							if (newy > this.limitHighPosY)
							{

								newy = this.limitHighPosY;

							}

						}

						//New Position

						if (this.inst.x !== newx || this.inst.y !== newy)	
							{					
								this.inst.x = newx;
								this.inst.y = newy;
								this.inst.set_bbox_changed();
				
							}

					}


				}
				
			}
			//If all Bound values are 0.
			else
			{


				if (this.inst.x !== newx || this.inst.y !== newy)	
				{					
					this.inst.x = newx;
					this.inst.y = newy;
					this.inst.set_bbox_changed();
				}

			}
			
		}
	
		else if (this.axes === 1)	// horizontal
		{				
			//If one or more Bound values aren't 0.
			if (this.boundEnabled)	
			{
				//origin
				if (this.boundBy === 0)
				{
					//Radius
					this.limitLowX = this.posX - this.boundXRadius;
					this.limitHighX = this.posX + this.boundXRadius;
					//Position
					this.limitLowPosX = this.boundMinX;
					this.limitHighPosX = this.boundMaxX;
					//Boolean
					var radiusClear = false;
					var posClear = false;

					//Set Clearance
					//radiusBound clearance
					if (this.radiusBound)
					{
						if (newx < this.limitLowX || newx > this.limitHighX)
						{
							radiusClear = false; //not necessary (redundant)

							//radius Exceeded
							this.radiusExceeded = true;

							//OnAnyBoundExceeded conditional trigger
							this.runtime.trigger(cr.behaviors.CV_BoundedDragnDrop.prototype.cnds.OnAnyBoundExceeded, this.inst);

							//OnRadiusExceeded conditional trigger
							this.runtime.trigger(cr.behaviors.CV_BoundedDragnDrop.prototype.cnds.OnRadiusExceeded, this.inst);


						}
						
						else 
						{

							radiusClear = true;

							//radius !Exceeded
							this.radiusExceeded = false;

						}
					
					}

					else
					{
						radiusClear = true;

						//radius !Exceeded
						this.radiusExceeded = false;

					}

					//posBound clearance
					if (this.posBound)
					{
						if (newx < this.limitLowPosX || newx > this.limitHighPosX)
						{
							posClear = false; //not necessary (redundant)

							//position Exceeded
							this.positionExceeded = true;

							//OnAnyBoundExceeded conditional trigger
							this.runtime.trigger(cr.behaviors.CV_BoundedDragnDrop.prototype.cnds.OnAnyBoundExceeded, this.inst);

							//OnPositionExceeded conditional trigger
							this.runtime.trigger(cr.behaviors.CV_BoundedDragnDrop.prototype.cnds.OnPositionExceeded, this.inst);

						}
						
						else 
						{

							posClear = true;

							//position !Exceeded
							this.positionExceeded = false;

						}
					
					}

					else
					{
						posClear = true;

						//position !Exceeded
						this.positionExceeded = false;

					}


					//Set New Position
					//BoundMethod = 'Halt'
					if (this.boundMethod === 0)
					{

						if (radiusClear && posClear)
						{
							if (this.inst.x !== newx)	
							{					
								this.inst.x = newx;
								this.inst.set_bbox_changed();
				
							}
						}

					}
					//BoundMethod = 'Clamp'
					else
					{
						//!RadiusClear
						if (!radiusClear)
						{
							//RadiusX
							if (newx < this.limitLowX)
							{

								newx = this.limitLowX;

							}
							
							if (newx > this.limitHighX)
							{

								newx = this.limitHighX;

							}
							//RadiusY
							if (newy < this.limitLowY)
							{

								newy = this.limitLowY;

							}
							
							if (newy > this.limitHighY)
							{

								newy = this.limitHighY;

							}

						}
						//!PosClear
						if (!posClear)
						{
							//PosX
							if (newx < this.limitLowPosX)
							{

								newx = this.limitLowPosX;

							}
							
							if (newx > this.limitHighPosX)
							{

								newx = this.limitHighPosX;

							}
							//PosY
							if (newy < this.limitLowPosY)
							{

								newy = this.limitLowPosY;

							}
							
							if (newy > this.limitHighPosY)
							{

								newy = this.limitHighPosY;

							}

						}

						//New Position

						if (this.inst.x !== newx)	
							{					
								this.inst.x = newx;
								this.inst.set_bbox_changed();
				
							}

					}

				}

				//edge
				else 

				{

					//Hotspot
					var hotspotX = this.inst.hotspotX;

					var hotspotWidthLeft = (hotspotX * this.inst.width);
					var hotspotWidthRight = ((1 - hotspotX) * this.inst.width);

					//Radius
					this.limitLowX = (this.posX + hotspotWidthLeft) - this.boundXRadius;
					this.limitHighX = (this.posX - hotspotWidthRight) + this.boundXRadius;
					//Position
					this.limitLowPosX = this.boundMinX + hotspotWidthLeft;
					this.limitHighPosX = this.boundMaxX - hotspotWidthRight;
					//Boolean
					var radiusClear = false;
					var posClear = false;

					//Set New Position

					
					//Set Clearance
					//radiusBound clearance
					if (this.radiusBound)
					{
						if (newx < this.limitLowX || newx > this.limitHighX)
						{
							radiusClear = false; //not necessary (redundant)

							//radius Exceeded
							this.radiusExceeded = true;

							//OnAnyBoundExceeded conditional trigger
							this.runtime.trigger(cr.behaviors.CV_BoundedDragnDrop.prototype.cnds.OnAnyBoundExceeded, this.inst);

							//OnRadiusExceeded conditional trigger
							this.runtime.trigger(cr.behaviors.CV_BoundedDragnDrop.prototype.cnds.OnRadiusExceeded, this.inst);

						}
						
						else 
						{

							radiusClear = true;

							//radius !Exceeded
							this.radiusExceeded = false;

						}
					
					}

					else
					{
						radiusClear = true;

						//radius !Exceeded
						this.radiusExceeded = false;

					}

					//posBound clearance
					if (this.posBound)
					{
						if (newx < this.limitLowPosX || newx > this.limitHighPosX)
						{
							posClear = false; //not necessary (redundant)
							
							//position Exceeded
							this.positionExceeded = true;

							//OnAnyBoundExceeded conditional trigger
							this.runtime.trigger(cr.behaviors.CV_BoundedDragnDrop.prototype.cnds.OnAnyBoundExceeded, this.inst);

							//OnPositionExceeded conditional trigger
							this.runtime.trigger(cr.behaviors.CV_BoundedDragnDrop.prototype.cnds.OnPositionExceeded, this.inst);
							

						}
						
						else 
						{

							posClear = true;

							//position !Exceeded
							this.positionExceeded = false;

						}
					
					}

					else
					{
						posClear = true;

						//position !Exceeded
						this.positionExceeded = false;

					}


					//Set New Position
					//BoundMethod = 'Halt'
					if (this.boundMethod === 0)
					{

						if (radiusClear && posClear)
						{
							if (this.inst.x !== newx)	
							{					
								this.inst.x = newx;
								this.inst.set_bbox_changed();
				
							}
						}

					}
					//BoundMethod = 'Clamp'
					else
					{
						//!RadiusClear
						if (!radiusClear)
						{
							//RadiusX
							if (newx < this.limitLowX)
							{

								newx = this.limitLowX;

							}
							
							if (newx > this.limitHighX)
							{

								newx = this.limitHighX;

							}
							//RadiusY
							if (newy < this.limitLowY)
							{

								newy = this.limitLowY;

							}
							
							if (newy > this.limitHighY)
							{

								newy = this.limitHighY;

							}

						}
						//!PosClear
						if (!posClear)
						{
							//PosX
							if (newx < this.limitLowPosX)
							{

								newx = this.limitLowPosX;

							}
							
							if (newx > this.limitHighPosX)
							{

								newx = this.limitHighPosX;

							}
							//PosY
							if (newy < this.limitLowPosY)
							{

								newy = this.limitLowPosY;

							}
							
							if (newy > this.limitHighPosY)
							{

								newy = this.limitHighPosY;

							}

						}

						//New Position

						if (this.inst.x !== newx)	
							{					
								this.inst.x = newx;
								this.inst.set_bbox_changed();
				
							}

					}


				}
				
			}
			//If all Bound values are 0.
			else
			{


				if (this.inst.x !== newx)
				{
					this.inst.x = newx;
					this.inst.set_bbox_changed();
				}
	

			}
			
		}

		else if (this.axes === 2)	// vertical
		{
			//If one or more Bound values aren't 0.
			if (this.boundEnabled)	
			{
				//origin
				if (this.boundBy === 0)
				{
					//Radius
					this.limitLowY = this.posY - this.boundYRadius;
					this.limitHighY = this.posY + this.boundYRadius;
					//Position
					this.limitLowPosY = this.boundMinY;
					this.limitHighPosY = this.boundMaxY;
					//Boolean
					var radiusClear = false;
					var posClear = false;

					//Set Clearance
					//radiusBound clearance
					if (this.radiusBound)
					{
						if (newy < this.limitLowY || newy > this.limitHighY)
						{
							radiusClear = false; //not necessary (redundant)

							//radius Exceeded
							this.radiusExceeded = true;

							//OnAnyBoundExceeded conditional trigger
							this.runtime.trigger(cr.behaviors.CV_BoundedDragnDrop.prototype.cnds.OnAnyBoundExceeded, this.inst);

							//OnRadiusExceeded conditional trigger
							this.runtime.trigger(cr.behaviors.CV_BoundedDragnDrop.prototype.cnds.OnRadiusExceeded, this.inst);


						}
						
						else 
						{

							radiusClear = true;

							//radius !Exceeded
							this.radiusExceeded = false;

						}
					
					}

					else
					{
						radiusClear = true;

						//radius !Exceeded
						this.radiusExceeded = false;

					}

					//posBound clearance
					if (this.posBound)
					{
						if (newy < this.limitLowPosY || newy > this.limitHighPosY)
						{
							posClear = false; //not necessary (redundant)

							//position Exceeded
							this.positionExceeded = true;

							//OnAnyBoundExceeded conditional trigger
							this.runtime.trigger(cr.behaviors.CV_BoundedDragnDrop.prototype.cnds.OnAnyBoundExceeded, this.inst);

							//OnPositionExceeded conditional trigger
							this.runtime.trigger(cr.behaviors.CV_BoundedDragnDrop.prototype.cnds.OnPositionExceeded, this.inst);

						}
						
						else 
						{

							posClear = true;

							//position !Exceeded
							this.positionExceeded = false;

						}
					
					}

					else
					{
						posClear = true;

						//position !Exceeded
						this.positionExceeded = false;

					}


					//Set New Position
					//BoundMethod = 'Halt'
					if (this.boundMethod === 0)
					{

						if (radiusClear && posClear)
						{
							if (this.inst.y !== newy)	
							{					
								this.inst.y = newy;
								this.inst.set_bbox_changed();
				
							}
						}

					}
					//BoundMethod = 'Clamp'
					else
					{
						//!RadiusClear
						if (!radiusClear)
						{
							//RadiusX
							if (newx < this.limitLowX)
							{

								newx = this.limitLowX;

							}
							
							if (newx > this.limitHighX)
							{

								newx = this.limitHighX;

							}
							//RadiusY
							if (newy < this.limitLowY)
							{

								newy = this.limitLowY;

							}
							
							if (newy > this.limitHighY)
							{

								newy = this.limitHighY;

							}

						}
						//!PosClear
						if (!posClear)
						{
							//PosX
							if (newx < this.limitLowPosX)
							{

								newx = this.limitLowPosX;

							}
							
							if (newx > this.limitHighPosX)
							{

								newx = this.limitHighPosX;

							}
							//PosY
							if (newy < this.limitLowPosY)
							{

								newy = this.limitLowPosY;

							}
							
							if (newy > this.limitHighPosY)
							{

								newy = this.limitHighPosY;

							}

						}

						//New Position

						if (this.inst.y !== newy)	
							{					
								this.inst.y = newy;
								this.inst.set_bbox_changed();
				
							}

					}
		

				}

				//edge
				else 

				{
					//Hotspot
					var hotspotY = this.inst.hotspotY;
									
					var hotspotHeightTop = (hotspotY * this.inst.height);
					var hotspotHeightBottom = ((1 - hotspotY) * this.inst.height);

					//Radius
					this.limitLowY = (this.posY + hotspotHeightTop) - this.boundYRadius;
					this.limitHighY = (this.posY - hotspotHeightBottom) + this.boundYRadius;
					//Position
					this.limitLowPosY = this.boundMinY + hotspotHeightTop;
					this.limitHighPosY = this.boundMaxY - hotspotHeightBottom;
					//Boolean
					var radiusClear = false;
					var posClear = false;

					//Set New Position

					
					//Set Clearance
					//radiusBound clearance
					if (this.radiusBound)
					{
						if (newy < this.limitLowY || newy > this.limitHighY)
						{
							radiusClear = false; //not necessary (redundant)

							//radius Exceeded
							this.radiusExceeded = true;

							//OnAnyBoundExceeded conditional trigger
							this.runtime.trigger(cr.behaviors.CV_BoundedDragnDrop.prototype.cnds.OnAnyBoundExceeded, this.inst);

							//OnRadiusExceeded conditional trigger
							this.runtime.trigger(cr.behaviors.CV_BoundedDragnDrop.prototype.cnds.OnRadiusExceeded, this.inst);

						}
						
						else 
						{

							radiusClear = true;

							//radius !Exceeded
							this.radiusExceeded = false;

						}
					
					}

					else
					{
						radiusClear = true;

						//radius !Exceeded
						this.radiusExceeded = false;

					}

					//posBound clearance
					if (this.posBound)
					{
						if (newy < this.limitLowPosY || newy > this.limitHighPosY)
						{
							posClear = false; //not necessary (redundant)

							//position Exceeded
							this.positionExceeded = true;

							//OnAnyBoundExceeded conditional trigger
							this.runtime.trigger(cr.behaviors.CV_BoundedDragnDrop.prototype.cnds.OnAnyBoundExceeded, this.inst);

							//OnPositionExceeded conditional trigger
							this.runtime.trigger(cr.behaviors.CV_BoundedDragnDrop.prototype.cnds.OnPositionExceeded, this.inst);

						}
						
						else 
						{

							posClear = true;

							//position !Exceeded
							this.positionExceeded = false;

						}
					
					}

					else
					{
						posClear = true;

						//position !Exceeded
						this.positionExceeded = false;

					}


					//Set New Position
					//BoundMethod = 'Halt'
					if (this.boundMethod === 0)
					{

						if (radiusClear && posClear)
						{
							if (this.inst.y !== newy)	
							{					
								this.inst.y = newy;
								this.inst.set_bbox_changed();
				
							}
						}

					}
					//BoundMethod = 'Clamp'
					else
					{
						//!RadiusClear
						if (!radiusClear)
						{
							//RadiusX
							if (newx < this.limitLowX)
							{

								newx = this.limitLowX;

							}
							
							if (newx > this.limitHighX)
							{

								newx = this.limitHighX;

							}
							//RadiusY
							if (newy < this.limitLowY)
							{

								newy = this.limitLowY;

							}
							
							if (newy > this.limitHighY)
							{

								newy = this.limitHighY;

							}

						}
						//!PosClear
						if (!posClear)
						{
							//PosX
							if (newx < this.limitLowPosX)
							{

								newx = this.limitLowPosX;

							}
							
							if (newx > this.limitHighPosX)
							{

								newx = this.limitHighPosX;

							}
							//PosY
							if (newy < this.limitLowPosY)
							{

								newy = this.limitLowPosY;

							}
							
							if (newy > this.limitHighPosY)
							{

								newy = this.limitHighPosY;

							}

						}

						//New Position

						if (this.inst.y !== newy)	
							{					
								this.inst.y = newy;
								this.inst.set_bbox_changed();
				
							}

					}


				}
			}
			//If all Bound values are 0.
			else
			{


				if (this.inst.y !== newy)	
				{					
					this.inst.y = newy;
					this.inst.set_bbox_changed();
				}

			}
		}

		//Chadori_RebornXD end

	};
	
	behinstProto.onUp = function()
	{
		this.dragging = false;
			
		// Trigger 'On drop'
		this.runtime.isInUserInputEvent = true;
		this.runtime.trigger(cr.behaviors.CV_BoundedDragnDrop.prototype.cnds.OnDrop, this.inst);
		this.runtime.isInUserInputEvent = false;
	};

	behinstProto.tick = function ()
	{
		//var dt = this.runtime.getDt(this.inst);
		
		// called every tick for you to update this.inst as necessary
		// dt is the amount of time passed since the last tick, in case it's a movement
	};
	
	/**BEGIN-PREVIEWONLY**/
	behinstProto.getDebuggerValues = function (propsections)
	{
		propsections.push({
			"title": this.type.name,
			"properties": [
				{"name": "Is dragging", "value": this.dragging, "readonly": true},
				{"name": "Enabled", "value": !!this.enabled}
			]
		});
	};
	
	behinstProto.onDebugValueEdited = function (header, name, value)
	{
		if (name === "Enabled")
			this.enabled = value;
	};
	/**END-PREVIEWONLY**/

	//////////////////////////////////////
	// Conditions
	function Cnds() {};

	Cnds.prototype.IsDragging = function ()
	{
		return this.dragging;
	};
	
	Cnds.prototype.OnDragStart = function ()
	{
		return true;
	};
	
	Cnds.prototype.OnDrop = function ()
	{
		return true;
	};
	
	Cnds.prototype.IsEnabled = function ()
	{
		return !!this.enabled;
	};

	//Chadori_RebornXD start

	Cnds.prototype.OnAnyBoundExceeded = function ()
	{
		return true;
	};

	Cnds.prototype.OnRadiusExceeded = function ()
	{
		return true;
	};

	Cnds.prototype.OnPositionExceeded = function ()
	{
		return true;
	};

	Cnds.prototype.IsRadiusExceeded = function ()
	{

		return this.radiusExceeded;
		
	};

	Cnds.prototype.IsPositionExceeded = function ()
	{

		return this.positionExceeded;
		
	};

	//Chadori_RebornXD end

	behaviorProto.cnds = new Cnds();

	//////////////////////////////////////
	// Actions
	function Acts() {};

	Acts.prototype.SetEnabled = function (s)
	{
		this.enabled = (s !== 0);
		
		// Got disabled: cancel any drag
		if (!this.enabled)
			this.dragging = false;
	};
	
	Acts.prototype.Drop = function ()
	{
		if (this.dragging)
			this.onUp();
	};

	//Chadori_RebornXD start
	
	Acts.prototype.SetBoundTo = function (boundTo)
	{
		this.boundTo = boundTo;

		this.getBoundValues();
	};

	Acts.prototype.SetBoundBy = function (boundBy)
	{
		this.boundBy = boundBy;

		this.getBoundValues();
	};

	Acts.prototype.SetBoundMethod = function (boundMethod)
	{
		this.boundMethod = boundMethod;

		//this.getBoundValues();
	};

	Acts.prototype.SetXYRadius = function (x, y)
	{
		this.boundXRadius = x;
		this.boundYRadius = y;

		this.getBoundValues();
	};

	Acts.prototype.SetXRadius = function (x)
	{
		this.boundXRadius = x;

		this.getBoundValues();
	};

	Acts.prototype.SetYRadius = function (y)
	{
		this.boundYRadius = y;

		this.getBoundValues();
	};

	Acts.prototype.SetBoundPosition = function (xI, xF, yI, yF)
	{
		this.boundMinX = xI;
		this.boundMaxX = xF;
		this.boundMinY = yI;
		this.boundMaxY = yF;

		this.getBoundValues();
	};

	Acts.prototype.SetBoundX = function (xI, xF)
	{
		this.boundMinX = xI;
		this.boundMaxX = xF;

		this.getBoundValues();
	};

	Acts.prototype.SetBoundY = function (yI, yF)
	{
		this.boundMinY = yI;
		this.boundMaxY = yF;

		this.getBoundValues();
	};

	Acts.prototype.OverwriteOrigin = function (x, y)
	{
		this.posX = x;
		this.posY = y;

		this.getBoundValues();
	};
	//Chadori_RebornXD end

	behaviorProto.acts = new Acts();

	//////////////////////////////////////
	// Expressions
	function Exps() {};

	//Chadori_RebornXD start
	
	Exps.prototype.BoundTo = function (ret)
	{

		var boundToString;

		switch(this.boundTo) 
		
		{

			case 0:
				
				boundToString = "Off";
				break;
			
			case 1 : 

				boundToString = "Both";
				break;

			case 2 : 

				boundToString = "Radius";
				break;
				
			case 3 : 

				boundToString = "Position";
		}


		ret.set_string(boundToString);
	};

	Exps.prototype.BoundBy = function (ret)
	{

		var boundByString;

		switch(this.boundBy) 
		
		{

			case 0:
				
				boundByString = "Origin";
				break;		
				
			case 1 : 

				boundByString = "Edge";

		}


		ret.set_string(boundByString);
	};

	Exps.prototype.BoundMethod = function (ret)
	{

		var boundMethodString;

		switch(this.boundMethod) 
		
		{

			case 0:
				
				boundMethodString = "Halt";
				break;		
				
			case 1 : 

				boundMethodString = "Clamp";

		}


		ret.set_string(boundMethodString);
	};


	Exps.prototype.xBoundRadius = function (ret)
	{
		ret.set_float(this.boundXRadius);
	};

	Exps.prototype.yBoundRadius = function (ret)
	{
		ret.set_float(this.boundYRadius);
	};

	Exps.prototype.xMinBoundPos = function (ret)
	{
		ret.set_float(this.boundMinX);
	};

	Exps.prototype.xMaxBoundPos = function (ret)
	{
		ret.set_float(this.boundMaxX);
	};

	Exps.prototype.yMinBoundPos = function (ret)
	{
		ret.set_float(this.boundMinY);
	};

	Exps.prototype.yMaxBoundPos = function (ret)
	{
		ret.set_float(this.boundMaxY);
	};

	Exps.prototype.xOrigin = function (ret)
	{
		ret.set_float(this.posX);
	};

	Exps.prototype.yOrigin = function (ret)
	{
		ret.set_float(this.posY);
	};


	//Chadori_RebornXD end


	behaviorProto.exps = new Exps();
	
}());